import os
import unittest
from xml.etree import ElementTree
import json
from elifecleaner import LOGGER, configure_logging, prc
from tests.helpers import delete_files_in_folder

# elife ISSN example of non-PRC journal-id tag values
NON_PRC_XML = (
    "<article><front><journal-meta>"
    '<journal-id journal-id-type="nlm-ta">elife</journal-id>'
    '<journal-id journal-id-type="hwp">eLife</journal-id>'
    '<journal-id journal-id-type="publisher-id">eLife</journal-id>'
    "<journal-title-group>"
    "<journal-title>eLife</journal-title>"
    "</journal-title-group>"
    '<issn pub-type="epub">2050-084X</issn>'
    "<publisher>"
    "<publisher-name>eLife Sciences Publications, Ltd</publisher-name>"
    "</publisher>"
    "</journal-meta></front></article>"
)

# PRC xml will have non-eLife journal-id tag text values
PRC_XML = (
    "<article><front><journal-meta>"
    '<journal-id journal-id-type="nlm-ta">foo</journal-id>'
    '<journal-id journal-id-type="hwp">foo</journal-id>'
    '<journal-id journal-id-type="publisher-id">foo</journal-id>'
    "<journal-title-group>"
    "<journal-title>eLife</journal-title>"
    "</journal-title-group>"
    '<issn pub-type="epub">2050-084X</issn>'
    "<publisher>"
    "<publisher-name>eLife Sciences Publications, Ltd</publisher-name>"
    "</publisher>"
    "</journal-meta></front></article>"
)


class TestIsXmlPrc(unittest.TestCase):
    def test_is_xml_prc(self):
        "PRC XML will return true"
        root = ElementTree.fromstring(PRC_XML)
        self.assertTrue(prc.is_xml_prc(root))

    def test_is_xml_prc_false(self):
        "test non-PRC XML will return false"
        root = ElementTree.fromstring(NON_PRC_XML)
        self.assertEqual(prc.is_xml_prc(root), False)

    def test_is_xml_prc_incomplete(self):
        "incomplete XML will return false"
        root = ElementTree.fromstring("<root/>")
        self.assertEqual(prc.is_xml_prc(root), False)


class TestTransformJournalIdTags(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_transform_journal_id_tags(self):
        # populate an ElementTree
        identifier = "test.zip"
        xml_string = PRC_XML
        expected = bytes(NON_PRC_XML, encoding="utf-8")
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.transform_journal_id_tags(root, identifier)
        # assertions
        self.assertEqual(ElementTree.tostring(root_output), expected)
        log_file_lines = []
        with open(self.log_file, "r") as open_file:
            for line in open_file:
                log_file_lines.append(line)
        for index, (journal_id_type, tag_text) in enumerate(
            [("nlm-ta", "elife"), ("hwp", "eLife"), ("publisher-id", "eLife")]
        ):
            self.assertEqual(
                log_file_lines[index],
                (
                    (
                        "INFO elifecleaner:prc:transform_journal_id_tags: "
                        "%s replacing journal-id tag text of type %s to %s\n"
                    )
                )
                % (identifier, journal_id_type, tag_text),
            )


class TestAddPrcCustomMetaTags(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.expected_xml = bytes(
            "<article>"
            "<front>"
            "<article-meta>"
            "<custom-meta-group>"
            '<custom-meta specific-use="meta-only">'
            "<meta-name>publishing-route</meta-name>"
            "<meta-value>prc</meta-value>"
            "</custom-meta>"
            "</custom-meta-group>"
            "</article-meta>"
            "</front>"
            "</article>",
            encoding="utf-8",
        )

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_add_custom_meta_tags(self):
        "test when custom-meta-group tag does not yet exist"
        # populate an ElementTree
        xml_string = "<article><front><article-meta/></front></article>"
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.add_prc_custom_meta_tags(root)
        # assertions
        self.assertEqual(ElementTree.tostring(root_output), self.expected_xml)

    def test_group_tag_exists(self):
        "test if custom-meta-group tag already exists"
        # populate an ElementTree
        xml_string = (
            "<article><front><article-meta>"
            "<custom-meta-group />"
            "</article-meta></front></article>"
        )
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.add_prc_custom_meta_tags(root)
        # assertions
        self.assertEqual(ElementTree.tostring(root_output), self.expected_xml)

    def test_no_article_meta_tag(self):
        # populate an ElementTree
        identifier = "test.zip"
        xml_string = "<root/>"
        expected = b"<root />"
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.add_prc_custom_meta_tags(root, identifier)
        # assertions
        self.assertEqual(ElementTree.tostring(root_output), expected)
        with open(self.log_file, "r") as open_file:
            self.assertEqual(
                open_file.read(),
                (
                    "WARNING elifecleaner:prc:add_prc_custom_meta_tags: "
                    "%s article-meta tag not found\n"
                )
                % identifier,
            )


class TestTransformElocationId(unittest.TestCase):
    def setUp(self):
        self.xml_string_pattern = (
            "<article><front><article-meta>%s</article-meta></front></article>"
        )

    def test_transform_elocation_id(self):
        xml_string = (
            self.xml_string_pattern % "<elocation-id>e1234567890</elocation-id>"
        )
        expected = bytes(
            self.xml_string_pattern % "<elocation-id>RP1234567890</elocation-id>",
            encoding="utf-8",
        )
        identifier = "test.zip"
        root = ElementTree.fromstring(xml_string)
        root_output = prc.transform_elocation_id(root, identifier=identifier)
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_no_change(self):
        xml_string = self.xml_string_pattern % "<elocation-id>foo</elocation-id>"
        expected = bytes(xml_string, encoding="utf-8")
        root = ElementTree.fromstring(xml_string)
        root_output = prc.transform_elocation_id(root)
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_tag_missing(self):
        xml_string = "<article />"
        expected = bytes(xml_string, encoding="utf-8")
        root = ElementTree.fromstring(xml_string)
        root_output = prc.transform_elocation_id(root)
        self.assertEqual(ElementTree.tostring(root_output), expected)


def docmap_test_data(doi=None):
    "generate a docmap json test fixture"
    docmap_json = {
        "first-step": "_:b0",
        "steps": {
            "_:b0": {"next-step": "_:b1"},
            "_:b1": {
                "actions": [
                    {
                        "outputs": [
                            {
                                "type": "preprint",
                                "identifier": "85111",
                                "versionIdentifier": "2",
                            }
                        ]
                    },
                ]
            },
        },
    }
    if doi:
        # add doi key and value to the outputs
        docmap_json["steps"]["_:b1"]["actions"][0]["outputs"][0]["doi"] = doi
    return docmap_json


class TestVersionDoiFromDocmap(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_version_doi_from_docmap(self):
        "test for when a doi is found"
        doi = "10.7554/eLife.85111.2"
        docmap_json = docmap_test_data(doi)
        result = prc.version_doi_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, doi)

    def test_docmap_is_none(self):
        "test for no docmap"
        docmap_json = {}
        result = prc.version_doi_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:version_doi_from_docmap: "
                    "%s parsing docmap returned None\n"
                )
                % self.identifier,
            )

    def test_no_preprint_in_docmap(self):
        "test for doi is not present"
        docmap_json = docmap_test_data(None)
        # delete the step holding the preprint data
        del docmap_json["steps"]["_:b1"]
        result = prc.version_doi_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:version_doi_from_docmap: "
                    "%s no preprint data was found in the docmap\n"
                )
                % self.identifier,
            )

    def test_no_doi_key_in_docmap(self):
        "test for doi is not present"
        docmap_json = docmap_test_data(None)
        result = prc.version_doi_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:version_doi_from_docmap: "
                    "%s did not find doi data in the docmap preprint data\n"
                )
                % self.identifier,
            )


class TestNextVersionDoi(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_next_version_doi(self):
        doi = "10.7554/eLife.85111.2"
        expected = "10.7554/eLife.85111.3"
        result = prc.next_version_doi(doi, self.identifier)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "INFO elifecleaner:prc:next_version_doi: "
                    "%s next version doi, from DOI %s, next DOI %s\n"
                )
                % (self.identifier, doi, expected),
            )

    def test_non_int_version(self):
        "non-int version value at the end"
        version = "sa1"
        doi = "10.7554/eLife.85111.2.%s" % version
        expected = None
        result = prc.next_version_doi(doi, self.identifier)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:next_version_doi: "
                    "%s version from DOI could not be converted to int, version %s\n"
                )
                % (self.identifier, version),
            )

    def test_version_exceeds_limit(self):
        "non-int version value at the end"
        article_id = "85111"
        doi = "10.7554/eLife.%s" % article_id
        expected = None
        result = prc.next_version_doi(doi, self.identifier)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:next_version_doi: "
                    "%s failed to determine the version from DOI, "
                    "version %s exceeds MAX_VERSION %s\n"
                )
                % (self.identifier, article_id, prc.MAX_VERSION),
            )

    def test_none(self):
        "non-int version value at the end"
        doi = None
        expected = None
        result = prc.next_version_doi(doi, self.identifier)
        self.assertEqual(result, expected)


class TestAddVersionDoi(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.doi = "10.7554/eLife.1234567890.5"
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_add_version_doi(self):
        xml_string = "<article><front><article-meta /></front></article>"
        root = ElementTree.fromstring(xml_string)
        expected = (
            b"<article>"
            b"<front>"
            b"<article-meta>"
            b'<article-id pub-id-type="doi" specific-use="version">'
            b"10.7554/eLife.1234567890.5"
            b"</article-id>"
            b"</article-meta>"
            b"</front>"
            b"</article>"
        )
        root_output = prc.add_version_doi(root, self.doi, self.identifier)
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_add_version_doi_in_order(self):
        "test the new article-id tag is added in a particular order"
        xml_string = (
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-id />"
            "<open-access>YES</open-access>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        expected = (
            b"<article>"
            b"<front>"
            b"<article-meta>"
            b"<article-id />"
            b'<article-id pub-id-type="doi" specific-use="version">'
            b"10.7554/eLife.1234567890.5"
            b"</article-id>"
            b"<open-access>YES</open-access>"
            b"</article-meta>"
            b"</front>"
            b"</article>"
        )
        root_output = prc.add_version_doi(root, self.doi, self.identifier)
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_no_article_meta(self):
        "test if no article-meta tag is in the XML"
        xml_string = "<article />"
        root = ElementTree.fromstring(xml_string)
        expected = bytes(xml_string, encoding="utf-8")
        root_output = prc.add_version_doi(root, self.doi, self.identifier)
        self.assertEqual(ElementTree.tostring(root_output), expected)
        with open(self.log_file, "r") as open_file:
            self.assertEqual(
                open_file.read(),
                (
                    "WARNING elifecleaner:prc:add_version_doi: "
                    "%s article-meta tag not found\n"
                )
                % self.identifier,
            )



import os
import unittest
import zipfile
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from xml.dom import minidom
from mock import patch
from elifearticle.article import (
    Affiliation,
    Article,
    Contributor,
    RelatedArticle,
    Role,
)
from elifetools import xmlio
from elifecleaner import configure_logging, LOGGER, parse, sub_article
from tests.helpers import delete_files_in_folder, read_fixture, read_log_file_lines


ARTICLE_TITLES = [
    b"Evaluation Summary: <italic>test</italic>",
    b"Reviewer #1 (Public Review):",
    b"Author Response:",
    b"Consensus Public Review:",
]

CONTENT_JSON = [
    {
        "type": "evaluation-summary",
        "html": b"<p><strong>%s</strong></p><p>Test evaluation summary.</p>"
        % ARTICLE_TITLES[0],
    },
    {
        "type": "review-article",
        "html": b"<p><strong>%s</strong></p><p>Test review article.</p>"
        % ARTICLE_TITLES[1],
    },
    {
        "type": "reply",
        "html": b"<p><strong>%s</strong></p><p>Test reply.</p>" % ARTICLE_TITLES[2],
    },
    {
        "type": "review-article",
        "html": b"<p><strong>%s</strong></p><p>Test consensus review article.</p>"
        % ARTICLE_TITLES[3],
    },
]


class FakeRequest:
    def __init__(self):
        self.headers = {}
        self.body = None


class FakeResponse:
    def __init__(self, status_code, response_json=None, text="", content=None):
        self.status_code = status_code
        self.response_json = response_json
        self.content = content
        self.text = text
        self.request = FakeRequest()
        self.headers = {}

    def json(self):
        return self.response_json


def editor_report_article_fixture(
    id_attribute="sa0",
    doi="10.7554/eLife.1234567890.4.sa0",
):
    "populate an editor-report Article for testing"
    article = Article(doi)
    article.article_type = "editor-report"
    article.title = "eLife assessment"
    article.id = id_attribute
    related_article = RelatedArticle()
    related_article.ext_link_type = "continued-by"
    related_article.xlink_href = (
        "https://sciety.org/articles/activity/10.1101/2021.11.09.467796"
    )
    article.related_articles = [related_article]
    # contributors
    author = Contributor("author", "Surname", "Given")
    author.suffix = "X"
    author.orcid = "https://orcid.org/0000-0000-0000-0000"
    author.orcid_authenticated = True
    # aff
    aff = Affiliation()
    aff.phone = "Phone"
    aff.fax = "Fax"
    aff.department = "Department"
    aff.institution = "Institution"
    aff.city = "City"
    aff.country = "Country"
    aff.ror = "ror"
    author.set_affiliation(aff)
    # add the author to the article
    article.add_contributor(author)
    return article


def referee_report_article_fixture(
    id_attribute="sa2",
    doi="10.7554/eLife.1234567890.4.sa2",
    title="Reviewer #1 (public review)",
):
    "populate a referee-report Article for testing"
    article = Article(doi)
    article.article_type = "referee-report"
    article.title = title
    article.id = id_attribute
    # contributors
    anonymous_author = Contributor("author", None, None)
    anonymous_author.roles = [Role("Reviewer", "referee")]
    # setattr(anonymous_author, "anonymous", True)
    anonymous_author.anonymous = True
    # add the author to the article
    article.add_contributor(anonymous_author)
    return article


def author_comment_article_fixture(
    id_attribute="sa3",
    doi="10.7554/eLife.1234567890.4.sa3",
):
    "populate an author-comment Article for testing"
    article = Article(doi)
    article.article_type = "author-comment"
    article.title = "Author response"
    article.id = id_attribute
    # contributors
    author = Contributor("author", "Surname", "Given")
    author.suffix = "X"
    article.add_contributor(author)
    return article


def xml_content(article_title):
    "return an XML content fixture with the article_title"
    return (
        b"<root><front-stub><title-group><article-title>%s"
        b"</article-title></title-group>\n</front-stub>"
        b"<body><p>The ....</p>\n</body>"
        b"</root>"
    ) % article_title


class TestReorderReviewArticles(unittest.TestCase):
    def test_reorder_review_articles(self):
        content_list = [
            {"xml": b"No match"},
            {"xml": xml_content(b"Reviewer #15 (Public Review):")},
            {"xml": xml_content(b"Reviewer #2 (Public Review):")},
            {"xml": xml_content(b"Reviewer #2 (Public Review):")},
            {"xml": xml_content(b"Reviewer #1 (Public Review):")},
            {"xml": xml_content(b"Reviewer #3 (Public Review):")},
        ]
        result = sub_article.reorder_review_articles(content_list)
        # assertions of the new sort order
        self.assertEqual(result[0], content_list[0])
        self.assertEqual(result[1], content_list[4])
        self.assertEqual(result[2], content_list[2])
        self.assertEqual(result[3], content_list[3])
        self.assertEqual(result[4], content_list[5])
        self.assertEqual(result[5], content_list[1])


class TestReorderContentJson(unittest.TestCase):
    def test_reorder_content_json(self):
        content_json = [
            {
                "type": "review-article",
                "xml": xml_content(b"Reviewer #2 (Public Review):"),
            },
            {"type": "reply", "xml": xml_content(b"Author Response:")},
            {"type": "evaluation-summary", "xml": xml_content(b"Evaluation Summary:")},
            {
                "type": "review-article",
                "xml": xml_content(b"Reviewer #1 (Public Review):"),
            },
        ]
        # type values in the expected order
        expected_types = [
            "evaluation-summary",
            "review-article",
            "review-article",
            "reply",
        ]
        result = sub_article.reorder_content_json(content_json)
        self.assertEqual([content.get("type") for content in result], expected_types)
        # assert the sorting by article-title was successful
        self.assertTrue(b"Reviewer #1" in result[1].get("xml"))


class TestAddSubArticleXml(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.original_repair_xml_value = parse.REPAIR_XML

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])
        parse.REPAIR_XML = self.original_repair_xml_value

    @patch("elifecleaner.sub_article.docmap_parse.populate_docmap_content")
    @patch("requests.get")
    def test_add_sub_article_xml(self, mock_get, mock_sub_article_data):
        mock_get.return_value = True
        mock_sub_article_data.return_value = CONTENT_JSON
        zip_file = "tests/test_data/08-11-2020-FA-eLife-64719.zip"
        xml_file_name = "08-11-2020-FA-eLife-64719/08-11-2020-FA-eLife-64719.xml"
        with zipfile.ZipFile(zip_file, "r") as input_zipfile:
            input_zipfile.extract(xml_file_name, self.temp_dir)
        docmap_string = read_fixture("2021.06.02.446694.docmap.json", mode="r")
        article_xml = os.path.join(self.temp_dir, xml_file_name)
        result = sub_article.add_sub_article_xml(docmap_string, article_xml)
        self.assertEqual(len(result.findall(".//sub-article")), 4)
        # For now assert each log file message appears, this may need adjustment later
        expected_log_file_lines = [
            "INFO elifecleaner:sub_article:add_sub_article_xml: Parsing article XML into root Element\n",
            "INFO elifecleaner:sub_article:add_sub_article_xml: Parsing article XML into an Article object\n",
            "INFO elifecleaner:sub_article:add_sub_article_xml: Populate sub article data\n",
            "INFO elifecleaner:sub_article:sub_article_data: Parsing docmap json\n",
            "INFO elifecleaner:sub_article:sub_article_data: Collecting content_json\n",
            "INFO elifecleaner:sub_article:sub_article_data: Downloading HTML for each web-content URL\n",
            "INFO elifecleaner:sub_article:sub_article_data: Formatting content json into article and XML data\n",
            "INFO elifecleaner:sub_article:add_sub_article_xml: Generate sub-article XML\n",
            "INFO elifecleaner:sub_article:add_sub_article_xml: Appending sub-article tags to the XML root\n",
        ]
        log_file_lines = read_log_file_lines(self.log_file)
        for line in expected_log_file_lines:
            self.assertTrue(line in log_file_lines)


class TestSubArticleData(unittest.TestCase):
    @patch("requests.get")
    def test_sub_article_data(self, mock_get):
        article_title = b"Evaluation Summary: <italic>test</italic>"
        mock_get.return_value = FakeResponse(
            200, content=b"<p><strong>%s</strong></p><p>Test.</p>" % article_title
        )
        docmap_string = read_fixture("2021.06.02.446694.docmap.json", mode="r")
        article = Article("10.7554/eLife.79713.1")
        # invoke
        sub_article_data = sub_article.sub_article_data(docmap_string, article)
        # assertions
        self.assertEqual(len(sub_article_data), 5)
        self.assertEqual(sorted(sub_article_data[0].keys()), ["article", "xml_root"])
        self.assertTrue(isinstance(sub_article_data[0].get("article"), Article))
        self.assertTrue(isinstance(sub_article_data[0].get("xml_root"), Element))
        # assertions for Article objects
        self.assertEqual(
            sub_article_data[0].get("article").article_type, "editor-report"
        )
        self.assertEqual(sub_article_data[0].get("article").doi, "%s.sa0" % article.doi)
        self.assertEqual(
            sub_article_data[0].get("article").title, article_title.decode("utf8")
        )
        # assertions for XML root
        self.assertIsNotNone(
            sub_article_data[0].get("xml_root").find(".//article-title")
        )


class TestSubArticleId(unittest.TestCase):
    def test_sub_article_id(self):
        index = 0
        expected = "sa0"
        self.assertEqual(sub_article.sub_article_id(index), expected)


class TestSubArticleDoi(unittest.TestCase):
    def test_sub_article_doi(self):
        article_doi = "10.7554/eLife.79713.1"
        index = 0
        self.assertEqual(
            sub_article.sub_article_doi(article_doi, index),
            "%s.sa%s" % (article_doi, index),
        )


class TestSubArticleContributors(unittest.TestCase):
    def setUp(self):
        self.article_object = Article()
        author = Contributor("author", "Surname", "Given")
        author.suffix = "X"
        author.orcid = "https://orcid.org/0000-0000-0000-0000"
        author.orcid_authenticated = True
        self.article_object.contributors.append(author)
        editor = Contributor("editor", "Itor", "Ed")
        self.article_object.editors.append(editor)
        # reviewer contrib_type will not be added to an editor-report
        reviewer = Contributor("reviewer", "Ewer", "Revi")
        self.article_object.editors.append(reviewer)

    def test_editor_report(self):
        "test editor-report article type"
        sub_article_object = Article()
        sub_article_object.article_type = "editor-report"
        sub_article.sub_article_contributors(self.article_object, sub_article_object)
        self.assertEqual(len(sub_article_object.contributors), 1)
        self.assertEqual(sub_article_object.contributors[0].given_name, "Ed")

    def test_referee_report(self):
        "test referee-report article type"
        sub_article_object = Article()
        sub_article_object.article_type = "referee-report"
        sub_article.sub_article_contributors(self.article_object, sub_article_object)
        self.assertEqual(len(sub_article_object.contributors), 1)
        # is anonymous
        self.assertEqual(sub_article_object.contributors[0].anonymous, True)

    def test_author_comment(self):
        "test author-comment article type"
        sub_article_object = Article()
        sub_article_object.article_type = "author-comment"
        sub_article.sub_article_contributors(self.article_object, sub_article_object)
        self.assertEqual(len(sub_article_object.contributors), 1)
        self.assertEqual(sub_article_object.contributors[0].given_name, "Given")

    def test_unsupported(self):
        "test unsupported article type"
        sub_article_object = Article()
        sub_article_object.article_type = "unsupported"
        sub_article.sub_article_contributors(self.article_object, sub_article_object)
        # no authors
        self.assertEqual(len(sub_article_object.contributors), 0)


class TestBuildSubArticleObject(unittest.TestCase):
    def test_build_sub_article_object(self):
        "test review-article example"
        article_object = Article("10.7554/eLife.79713.1")
        xml_root = ElementTree.fromstring(
            b"<article><front-stub><title-group>"
            b"<article-title>Title</article-title>"
            b"</title-group></front-stub></article>"
        )
        content = {"type": "review-article"}
        index = 1
        sub_article_object = sub_article.build_sub_article_object(
            article_object, xml_root, content, index
        )
        self.assertEqual(sub_article_object.doi, "10.7554/eLife.79713.1.sa1")
        self.assertEqual(sub_article_object.title, "Title")
        self.assertEqual(len(sub_article_object.contributors), 1)

    def test_minimal(self):
        "test minimal, incomplete, arguments supplied"
        article_object = Article("10.7554/eLife.79713.1")
        xml_root = Element("root")
        content = {}
        index = 0
        sub_article_object = sub_article.build_sub_article_object(
            article_object, xml_root, content, index
        )
        self.assertEqual(sub_article_object.doi, "10.7554/eLife.79713.1.sa0")
        self.assertEqual(sub_article_object.title, None)
        self.assertEqual(len(sub_article_object.contributors), 0)


class TestFormatContentJson(unittest.TestCase):
    def test_format_content_json(self):
        article_titles = [
            b"Evaluation Summary: <italic>test</italic>",
            b"Reviewer #1 (Public Review):",
            b"Author Response:",
        ]
        article = Article("10.7554/eLife.79713.1")
        article.contributors = [Contributor("author", "Surname", "Given")]
        article.editors = [Contributor("editor", "Editor", "Given")]
        # invoke
        sub_article_data = sub_article.format_content_json(CONTENT_JSON, article)
        # assertions
        self.assertEqual(len(sub_article_data), 4)
        self.assertEqual(sorted(sub_article_data[0].keys()), ["article", "xml_root"])
        self.assertTrue(isinstance(sub_article_data[0].get("article"), Article))
        self.assertTrue(isinstance(sub_article_data[0].get("xml_root"), Element))
        # assertions for Article objects
        article_0 = sub_article_data[0].get("article")
        self.assertEqual(article_0.article_type, "editor-report")
        self.assertEqual(article_0.doi, "%s.sa0" % article.doi)
        self.assertEqual(article_0.title, article_titles[0].decode("utf8"))
        # article 0 contributors
        self.assertEqual(len(article_0.contributors), 1)
        # article 0 contrib 0
        self.assertEqual(
            article_0.contributors[0].roles[0].text,
            "Reviewing Editor",
        )
        self.assertEqual(
            article_0.contributors[0].roles[0].specific_use,
            "editor",
        )
        # article 1
        article_1 = sub_article_data[1].get("article")
        self.assertEqual(article_1.article_type, "referee-report")
        # article 1 contributors
        self.assertEqual(len(article_1.contributors), 1)
        # article 1 contrib 0
        self.assertEqual(article_1.contributors[0].roles[0].text, "Reviewer")
        self.assertEqual(
            article_1.contributors[0].roles[0].specific_use,
            "referee",
        )

        # article 2
        article_2 = sub_article_data[2].get("article")
        self.assertEqual(article_2.article_type, "referee-report")
        # article 2 contributors
        self.assertEqual(len(article_2.contributors), 1)
        # article 2 contrib 0
        self.assertEqual(article_2.contributors[0].roles[0].text, "Reviewer")
        self.assertEqual(
            article_2.contributors[0].roles[0].specific_use,
            "referee",
        )

        # article 3
        article_3 = sub_article_data[3].get("article")
        self.assertEqual(article_3.article_type, "author-comment")
        # article 3 contributors
        self.assertEqual(len(article_3.contributors), 1)
        # article 3 contrib 0
        self.assertEqual(article_3.contributors[0].roles[0].text, "Author")
        self.assertEqual(
            article_3.contributors[0].roles[0].specific_use,
            "author",
        )

        # assertions for XML root
        body_tag = sub_article_data[0].get("xml_root").find(".//body")
        self.assertIsNotNone(body_tag)
        self.assertEqual(body_tag.find("p").text, "Test evaluation summary.")


class TestGenerate(unittest.TestCase):
    def test_generate(self):
        editor_report_article = editor_report_article_fixture()
        referee_report_article = referee_report_article_fixture()
        author_comment_article = author_comment_article_fixture()

        # XML generated from docmap-tools
        # register XML namespaces
        xmlio.register_xmlns()
        editor_report_xml_string = (
            '<root xmlns:mml="http://www.w3.org/1998/Math/MathML">'
            "<body>"
            "<p><bold>Test</bold> <sup>superscript</sup> <sub>subscript</sub> &amp; p&lt;0.001 "
            "<italic>C. elegans</italic></p>"
            "<p><inline-formula>"
            '<mml:math display="inline" alttext="n"><mml:mi>n</mml:mi></mml:math>'
            "</inline-formula></p>"
            "</body></root>"
        )
        editor_report_sub_article_root = ElementTree.fromstring(
            editor_report_xml_string
        )
        referee_report_xml_string = "<root><body><p>Review.</p></body></root>"
        referee_report_sub_article_root = ElementTree.fromstring(
            referee_report_xml_string
        )
        author_comment_xml_string = (
            "<root>"
            "<body>"
            '<disp-quote content-type="editor-comment">Quotation.</disp-quote>'
            "<p>Response.</p>"
            "</body>"
            "</root>"
        )
        author_comment_sub_article_root = ElementTree.fromstring(
            author_comment_xml_string
        )

        # assemble sub article data
        sub_article_1 = {
            "article": editor_report_article,
            "xml_root": editor_report_sub_article_root,
        }
        sub_article_2 = {
            "article": referee_report_article,
            "xml_root": referee_report_sub_article_root,
        }
        sub_article_3 = {
            "article": author_comment_article,
            "xml_root": author_comment_sub_article_root,
        }
        # build XML
        sub_article_data = [sub_article_1, sub_article_2, sub_article_3]
        root = sub_article.generate(sub_article_data)
        expected = read_fixture("sub_articles.xml", mode="rb")

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        # parse the XML string to produce pretty output and
        # also check for XML namespace parsing errors
        reparsed = minidom.parseString(rough_xml_string)
        pretty_xml_string = reparsed.toprettyxml(indent="  ", encoding="utf-8")

        self.assertEqual(pretty_xml_string, expected)

    def test_editor_report(self):
        "test editor-report example with specific terms in the abstract"
        editor_report_article = Article("10.7554/eLife.79713.1")
        editor_report_article.article_type = "editor-report"
        editor_report_article.title = "eLife assessment"
        editor_report_xml_string = (
            "<root>"
            "<body>"
            "<p><strong>eLife assessment</strong></p>"
            "<p>Landmark convincing evaluation, "
            "convincingly compelling if "
            "incompletely.</p>"
            "<p>And important.</p>"
            "</body></root>"
        )
        editor_report_sub_article_root = ElementTree.fromstring(
            editor_report_xml_string
        )
        # assemble sub article data
        sub_article_1 = {
            "article": editor_report_article,
            "xml_root": editor_report_sub_article_root,
        }

        root = sub_article.generate([sub_article_1])
        expected = (
            b'<article><sub-article article-type="editor-report">'
            b"<front-stub>"
            b'<article-id pub-id-type="doi">10.7554/eLife.79713.1</article-id>'
            b"<title-group><article-title>eLife assessment</article-title></title-group>"
            b'<kwd-group kwd-group-type="evidence-strength">'
            b"<kwd>Compelling</kwd>"
            b"<kwd>Convincing</kwd>"
            b"<kwd>Incomplete</kwd>"
            b"</kwd-group>"
            b'<kwd-group kwd-group-type="claim-importance">'
            b"<kwd>Important</kwd>"
            b"<kwd>Landmark</kwd>"
            b"</kwd-group>"
            b"</front-stub>"
            b"<body>"
            b"<p><strong>eLife assessment</strong></p>"
            b"<p><bold>Landmark</bold> <bold>convincing</bold> evaluation, "
            b"<bold>convincingly</bold> <bold>compelling</bold> if "
            b"<bold>incompletely</bold>.</p>"
            b"<p>And <bold>important</bold>.</p>"
            b"</body>"
            b"</sub-article>"
            b"</article>"
        )
        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(rough_xml_string, expected)



import re
import shutil
import subprocess


def pdfimages_exists():
    "check if pdfimages exists"
    return shutil.which("pdfimages")


def pdfimages_output(pdf):
    "invoke pdfimages utility"
    return subprocess.run(
        ["pdfimages", "-list", pdf],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        check=False,
    )


def pdf_image_pages(pdf):
    "return pdf pages on which images are found from pdfimages output"
    result = pdfimages_output(pdf)
    page_list = []
    if result.stdout:
        page_line_match_pattern = re.compile(r"\s+(\d+)\s.*")
        output_lines = str(result.stdout, encoding="utf8").split("\n")
        if output_lines and output_lines[0].startswith("page "):
            for line in output_lines:
                match_result = re.match(page_line_match_pattern, line)
                if match_result:
                    page_list.append(int(match_result.group(1)))
    # de-dupe page list into a set of unique values
    return set(page_list)



import copy
from xml.etree.ElementTree import SubElement
import os
import zipfile
from elifetools import xmlio
from elifetools import parseJATS as parser
from elifecleaner import LOGGER, parse, prc, zip_lib


WELLCOME_FUNDING_STATEMENT = "For the purpose of Open Access, the authors have applied a CC BY public copyright license to any Author Accepted Manuscript version arising from this submission."


class ArticleZipFile:
    "data structure for holding details about files in a zip with a manifest XML"

    def __init__(self, xml_name=None, zip_name=None, file_path=None):
        self.xml_name = xml_name
        self.zip_name = zip_name
        self.file_path = file_path

    def __repr__(self):
        return 'ArticleZipFile("%s", "%s", "%s")' % (
            self.xml_name,
            self.zip_name,
            self.file_path,
        )


def transform_ejp_zip(zip_file, tmp_dir, output_dir):
    "transform ejp zip file and write a new zip file output"

    zip_file_name = zip_file.split(os.sep)[-1]

    # profile the zip contents
    asset_file_name_map = zip_lib.unzip_zip(zip_file, tmp_dir)

    # start logging
    LOGGER.info("%s starting to transform", zip_file_name)

    new_asset_file_name_map = transform_ejp_files(
        asset_file_name_map, output_dir, zip_file_name
    )

    # write new zip file
    new_zip_file_path = rezip(new_asset_file_name_map, output_dir, zip_file_name)

    return new_zip_file_path


def transform_ejp_files(asset_file_name_map, output_dir, identifier):
    "transform ejp files and XML"
    xml_asset = parse.article_xml_asset(asset_file_name_map)
    xml_asset_path = xml_asset[1]

    new_asset_file_name_map = transform_code_files(
        asset_file_name_map, output_dir, identifier
    )

    transform_xml(xml_asset_path, identifier)
    return new_asset_file_name_map


def transform_code_files(asset_file_name_map, output_dir, identifier):
    "zip code files if they are not already a zip file"
    # parse XML file
    xml_asset = parse.article_xml_asset(asset_file_name_map)
    xml_asset_path = xml_asset[1]
    root = parse.parse_article_xml(xml_asset_path)

    file_transformations = code_file_transformations(
        root, asset_file_name_map, output_dir, identifier
    )
    code_file_zip(file_transformations, output_dir, identifier)

    # create a new asset map
    new_asset_file_name_map = transform_asset_file_name_map(
        asset_file_name_map, file_transformations
    )

    xml_rewrite_file_tags(xml_asset_path, file_transformations, identifier)
    return new_asset_file_name_map


def code_file_transformations(root, asset_file_name_map, output_dir, identifier):
    # zip code files
    code_files = code_file_list(root)
    file_transformations = []
    for file_data in code_files:
        code_file_name = file_data.get("upload_file_nm")

        LOGGER.info("%s code_file_name: %s", identifier, code_file_name)
        # collect file name data
        original_code_file_name, original_code_file_path = find_in_file_name_map(
            code_file_name, asset_file_name_map
        )

        from_file = ArticleZipFile(
            code_file_name, original_code_file_name, original_code_file_path
        )
        LOGGER.info("%s from_file: %s", identifier, from_file)

        to_file = zip_code_file(from_file, output_dir)
        LOGGER.info("%s to_file: %s", identifier, to_file)

        # save the from file to file transformation
        file_transformations.append((from_file, to_file))
    return file_transformations


def code_file_zip(file_transformations, output_dir, identifier):
    for from_file, to_file in file_transformations:
        LOGGER.info(
            "%s zipping from_file: %s, to_file: %s", identifier, from_file, to_file
        )
        to_file = zip_code_file(from_file, output_dir)


def xml_rewrite_file_tags(xml_asset_path, file_transformations, identifier):
    root = parse.parse_article_xml(xml_asset_path)
    # rewrite the XML tags
    LOGGER.info("%s rewriting xml tags", identifier)
    root = transform_xml_file_tags(root, file_transformations)
    write_xml_file(root, xml_asset_path, identifier)


def transform_xml(xml_asset_path, identifier):
    "modify the XML"
    # remove history tags from XML for certain article types
    root = parse.parse_article_xml(xml_asset_path)
    soup = parser.parse_document(xml_asset_path)
    root = transform_xml_history_tags(root, soup, identifier)
    root = transform_xml_funding(root, identifier)
    write_xml_file(root, xml_asset_path, identifier)


def write_xml_file(root, xml_asset_path, identifier):
    # write new XML file
    xml_string = xml_element_to_string(root)
    LOGGER.info("%s writing xml to file %s", identifier, xml_asset_path)
    with open(xml_asset_path, "w") as open_file:
        open_file.write(xml_string)


def xml_element_to_string(root):
    xmlio.register_xmlns()
    return xmlio.output_root(root, None, None)


def code_file_list(root):
    "get a list of code files from the file tags in the ElementTree"
    code_files = []

    files = parse.file_list(root)

    aux_files = [
        file_data for file_data in files if file_data.get("file_type") == "aux_file"
    ]

    for file_data in aux_files:
        if file_data.get("upload_file_nm").endswith(".zip"):
            # if it is already a zip file, skip it
            continue
        code_files.append(file_data)

    return code_files


def find_in_file_name_map(file_name, file_name_map):
    "find the item in the map matching the file_name"
    for asset_file_name, file_name_path in file_name_map.items():
        if file_name_path.endswith(file_name):
            return asset_file_name, file_name_path
    return None, None


def zip_code_file(from_file, output_dir):
    "zip a code file and put new zip details into an ArticleZipFile struct"
    code_file_zip_name = from_file.zip_name + ".zip"
    new_code_file_name = from_file.xml_name + ".zip"
    code_file_zip_path = os.path.join(output_dir, new_code_file_name)

    to_file = ArticleZipFile(new_code_file_name, code_file_zip_name, code_file_zip_path)

    with zipfile.ZipFile(code_file_zip_path, "w") as open_zipfile:
        open_zipfile.write(from_file.file_path, from_file.zip_name)

    return to_file


def from_file_to_file_map(file_transformations):
    "convert a list of file transformations into a dict keyed on their xml file name"
    return {
        from_file.xml_name: to_file.xml_name
        for from_file, to_file in file_transformations
    }


def transform_xml_file_tags(root, file_transformations):
    "replace file name tags in xml Element with names from file transformations list"
    xml_file_name_transforms = from_file_to_file_map(file_transformations)
    for file_nm_tag in root.findall("./front/article-meta/files/file/upload_file_nm"):
        if file_nm_tag.text in xml_file_name_transforms:
            file_nm_tag.text = xml_file_name_transforms.get(file_nm_tag.text)
    return root


def transform_xml_history_tags(root, soup, zip_file_name):
    "remove history tags from the XML for particular article types"
    article_type = parser.article_type(soup)
    display_channel_list = parser.display_channel(soup)
    LOGGER.info(
        "%s article_type %s, display_channel %s",
        zip_file_name,
        article_type,
        display_channel_list,
    )

    if (
        article_type in ["correction", "editorial", "retraction"]
        or (
            article_type == "article-commentary"
            and "insight" in [value.lower() for value in display_channel_list if value]
        )
        or prc.is_xml_prc(root)
    ):
        LOGGER.info("%s transforming xml history tags", zip_file_name)
        # remove history tag
        for history_tag in root.findall("./front/article-meta/history"):
            root.find("./front/article-meta").remove(history_tag)
    return root


def transform_xml_funding(root, zip_file_name):
    "alter the content in the XML <funding-group> tags"
    # get funder names
    funding_sources = []
    for funding_source_tag in root.findall(
        ".//front/article-meta/funding-group/award-group/funding-source"
    ):
        funding_sources.append(funding_source_tag.text)
    LOGGER.info(
        "%s funding_sources %s",
        zip_file_name,
        funding_sources,
    )
    # look for Wellcome term
    wellcome = bool(
        [funding for funding in funding_sources if "wellcome" in funding.lower()]
    )

    add_funding_statement = False
    funding_statement_tag = None
    funding_statement = None

    if wellcome is True:
        # if Wellcome then look for funding-statement sentence
        LOGGER.info(
            "%s wellcome term found in funding_sources",
            zip_file_name,
        )

        funding_statement_tag = root.find(
            ".//front/article-meta/funding-group/award-group/funding-statement"
        )

        if funding_statement_tag:
            # use the text from the first tag found only
            funding_statement = funding_statement_tag.text
        if WELLCOME_FUNDING_STATEMENT not in str(funding_statement):
            add_funding_statement = True

    # if the sentence is not in funding-statement, then add to the funding statement
    if add_funding_statement:
        if not funding_statement_tag:
            # if no funding-statement tag is present, add it
            LOGGER.info(
                "%s adding a funding-statement tag to the first award-group tag",
                zip_file_name,
            )
            first_award_group_tag = root.find(
                ".//front/article-meta/funding-group/award-group"
            )
            if first_award_group_tag:
                funding_statement_tag = SubElement(
                    first_award_group_tag, "funding-statement"
                )

        # modify the funding-statement tag text
        LOGGER.info(
            "%s adding the WELLCOME_FUNDING_STATEMENT to the funding-statement",
            zip_file_name,
        )
        funding_statement_tag.text = ". ".join(
            [
                part
                for part in [funding_statement_tag.text, WELLCOME_FUNDING_STATEMENT]
                if part
            ]
        )

    return root


def transform_asset_file_name_map(asset_file_name_map, file_transformations):
    "replace file name details in the map with those from the list of file transformations"
    new_asset_file_name_map = copy.copy(asset_file_name_map)
    for from_file, to_file in file_transformations:
        if from_file.zip_name in new_asset_file_name_map:
            del new_asset_file_name_map[from_file.zip_name]
            new_asset_file_name_map[to_file.zip_name] = to_file.file_path
    return new_asset_file_name_map


def rezip(asset_file_name_map, output_dir, zip_file_name):
    "write new zip file"
    new_zip_file_path = os.path.join(output_dir, zip_file_name)
    LOGGER.info("%s writing new zip file %s", zip_file_name, new_zip_file_path)
    create_zip_from_file_map(new_zip_file_path, asset_file_name_map)
    return new_zip_file_path


def create_zip_from_file_map(zip_path, file_name_map):
    "write the files to a zip"
    with zipfile.ZipFile(
        zip_path, "w", zipfile.ZIP_DEFLATED, allowZip64=True
    ) as open_zip:
        for file_name, file_path in file_name_map.items():
            open_zip.write(file_path, file_name)



import unittest
from collections import OrderedDict
from elifecleaner import zip_lib
from tests.helpers import delete_files_in_folder, read_fixture


class TestZipLib(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_unzip_zip(self):
        zip_file = "tests/test_data/30-01-2019-RA-eLife-45644.zip"
        asset_file_name_map = zip_lib.unzip_zip(zip_file, self.temp_dir)
        expected = read_fixture("asset_file_name_map_45644.py")
        self.assertEqual(asset_file_name_map, expected)



import time
from xml.dom import minidom
from xml.etree import ElementTree
from xml.etree.ElementTree import Element, SubElement
from elifearticle.article import ArticleDate
from elifecleaner import parse
from elifecleaner.utils import pad_msid

JOURNAL_ID_TYPES = ["nlm-ta", "publisher-id"]

NAMESPACE_MAP = {
    "xmlns:mml": "http://www.w3.org/1998/Math/MathML",
    "xmlns:xlink": "http://www.w3.org/1999/xlink",
}

MEDIA_CONTENT_TYPE = "glencoe play-in-place height-250 width-310"

DTD_VERSION = "1.1d1"


def set_pub_date(parent, article, pub_type):
    "set XML pub-date tag to values from an article date"
    pub_date = article.get_date(pub_type)
    if pub_date:
        date_tag = SubElement(parent, "pub-date")
        date_tag.set("date-type", pub_type)
        if pub_date.publication_format:
            date_tag.set("publication-format", pub_date.publication_format)
        day = SubElement(date_tag, "day")
        day.text = str(pub_date.date.tm_mday).zfill(2)
        month = SubElement(date_tag, "month")
        month.text = str(pub_date.date.tm_mon).zfill(2)
        year = SubElement(date_tag, "year")
        year.text = str(pub_date.date.tm_year)


def set_article_meta(parent, article):
    "set XML article-meta tag and tags found inside it"
    article_meta = SubElement(parent, "article-meta")

    # article-id pub-id-type="publisher-id"
    if article.manuscript:
        pub_id_type = "publisher-id"
        article_id = SubElement(article_meta, "article-id")
        article_id.text = pad_msid(article.manuscript)
        article_id.set("pub-id-type", pub_id_type)

    # article-id pub-id-type="doi"
    if article.doi:
        pub_id_type = "doi"
        article_id = SubElement(article_meta, "article-id")
        article_id.text = article.doi
        article_id.set("pub-id-type", pub_id_type)

    set_pub_date(article_meta, article, "publication")


def set_front(parent, journal_data, article):
    "set XML front tag and the tags found inside it"
    front_tag = SubElement(parent, "front")
    journal_meta = SubElement(front_tag, "journal-meta")

    # journal-id
    if journal_data.get("journal_ids"):
        for journal_id_type, value in journal_data.get("journal_ids").items():
            # concatenate the config name to look for the value
            if journal_id_type:
                journal_id = SubElement(journal_meta, "journal-id")
                journal_id.set("journal-id-type", journal_id_type)
                journal_id.text = value
    # journal-title-group
    journal_title_group = SubElement(journal_meta, "journal-title-group")

    # journal-title
    if journal_data.get("journal_title"):
        journal_title_tag = SubElement(journal_title_group, "journal-title")
        journal_title_tag.text = journal_data.get("journal_title")

    # publisher
    if journal_data.get("publisher_name"):
        publisher = SubElement(journal_meta, "publisher")
        publisher_name_tag = SubElement(publisher, "publisher-name")
        publisher_name_tag.text = journal_data.get("publisher_name")

    set_article_meta(front_tag, article)


def set_body(parent, video_data):
    "set XML body tag contents"
    body_tag = SubElement(parent, "body")
    # video media tags
    for video in video_data:
        media_tag = SubElement(body_tag, "media")
        media_tag.set("xlink:href", video.get("video_filename"))
        media_tag.set("id", video.get("video_id"))
        media_tag.set("content-type", MEDIA_CONTENT_TYPE)
        media_tag.set("mimetype", "video")


def output_xml(root, pretty=False, indent=""):
    "output root XML Element to a string"
    encoding = "utf-8"
    rough_string = ElementTree.tostring(root, encoding)
    reparsed = minidom.parseString(rough_string)

    if pretty is True:
        return reparsed.toprettyxml(indent, encoding=encoding)
    return reparsed.toxml(encoding=encoding)


def generate_xml(article, journal_data, video_data, pretty=True, indent=""):
    "from jats_content generate final JATS output"

    # set the date to today
    article_date = ArticleDate("publication", time.gmtime())
    article_date.publication_format = "electronic"
    article.add_date(article_date)

    # Create the root XML node
    root = Element("article")
    # set attributes
    root.set("dtd-version", DTD_VERSION)
    root.set("article-type", article.article_type)
    # set namespaces
    for prefix, url in NAMESPACE_MAP.items():
        root.set(prefix, url)

    set_front(root, journal_data, article)
    set_body(root, video_data)

    return output_xml(root, pretty, indent)


def glencoe_xml(xml_file_path, video_data, pretty=True, indent=""):
    "generate XML to be submitted to Glencoe"
    # build an Article object from the XML
    article, error_count = parse.article_from_xml(xml_file_path)
    # collect journal data from the XML elementtree
    root = parse.parse_article_xml(xml_file_path)
    journal_ids = parse.xml_journal_id_values(root)
    filtered_journal_ids = {
        key: value for key, value in journal_ids.items() if key in JOURNAL_ID_TYPES
    }
    journal_title = parse.xml_journal_title(root)
    publisher_name = parse.xml_publisher_name(root)
    journal_data = {
        "journal_ids": filtered_journal_ids,
        "journal_title": journal_title,
        "publisher_name": publisher_name,
    }
    return generate_xml(article, journal_data, video_data, pretty, indent)



import copy
import re
from xml.etree import ElementTree
from xml.etree.ElementTree import Element, SubElement
from elifearticle.article import Article, Contributor, Role
from docmaptools import parse as docmap_parse
from jatsgenerator import build
from elifecleaner import assessment_terms, LOGGER, parse

XML_NAMESPACES = {
    "ali": "http://www.niso.org/schemas/ali/1.0/",
    "mml": "http://www.w3.org/1998/Math/MathML",
    "xlink": "http://www.w3.org/1999/xlink",
}

# to replace docmap article type with sub-article article_type
ARTICLE_TYPE_MAP = {
    "evaluation-summary": "editor-report",
    "review-article": "referee-report",
    "reply": "author-comment",
}


def reorder_review_articles(content_list):
    "reorder content based on the article-title, if present"
    number_match = re.compile(rb".*<article-title>.*\s+#(\d+)\s+.*")
    content_to_sort = []
    for content in content_list:
        print(content.get("xml"))
        matches = number_match.match(content.get("xml"))
        if matches:
            content_map = {"num": int(matches[1]), "content": content}
        else:
            content_map = {"num": 0, "content": content}
        content_to_sort.append(content_map)
    sorted_content = sorted(content_to_sort, key=lambda item: item.get("num"))
    return [content_item.get("content") for content_item in sorted_content]


def reorder_content_json(content_json):
    "reorder the content list"
    # append lists of specific content types to make a new list
    content_json = (
        [
            content
            for content in content_json
            if content.get("type") == "evaluation-summary"
        ]
        + reorder_review_articles(
            [
                content
                for content in content_json
                if content.get("type") not in ["evaluation-summary", "reply"]
            ]
        )
        + [content for content in content_json if content.get("type") == "reply"]
    )
    return content_json


def add_sub_article_xml(docmap_string, article_xml):
    "parse content from docmap and add sub-article tags to the article XML"
    LOGGER.info("Parsing article XML into root Element")
    root = parse.parse_article_xml(article_xml)
    LOGGER.info("Parsing article XML into an Article object")
    article, error_count = parse.article_from_xml(article_xml)
    LOGGER.info("Populate sub article data")
    data = sub_article_data(docmap_string, article)
    LOGGER.info("Generate sub-article XML")
    sub_article_xml_root = generate(data)
    LOGGER.info("Appending sub-article tags to the XML root")
    for sub_article_tag in sub_article_xml_root.findall(".//sub-article"):
        root.append(sub_article_tag)
    return root


def sub_article_data(docmap_string, article):
    "parse docmap, get the HTML for each article, and format the content"
    LOGGER.info("Parsing docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    LOGGER.info("Collecting content_json")
    content_json = docmap_parse.docmap_content(d_json)
    LOGGER.info("Downloading HTML for each web-content URL")
    content_json = docmap_parse.populate_docmap_content(content_json)
    LOGGER.info("Formatting content json into article and XML data")
    return format_content_json(content_json, article)


def sub_article_id(index):
    "generate an id attribute for a sub article"
    return "sa%s" % index


def sub_article_doi(article_doi, index):
    "generate a DOI for a sub article"
    return "%s.%s" % (article_doi, sub_article_id(index))


EDITOR_REPORT_CONTRIB_TYPES = ["editor"]


def sub_article_contributors(article_object, sub_article_object):
    "add contributors to the sub-article from the parent article depending on the article type"
    if sub_article_object.article_type == "editor-report":
        # add editors of the article as authors of the sub-article
        for editor in article_object.editors:
            if editor.contrib_type not in EDITOR_REPORT_CONTRIB_TYPES:
                continue
            author = copy.copy(editor)
            author.contrib_type = "author"
            if not author.roles:
                author.roles = [Role("Reviewing Editor", "editor")]
            sub_article_object.contributors.append(author)
    if sub_article_object.article_type == "referee-report":
        # one anonymous author per referee-report
        anonymous_author = Contributor("author", None, None)
        anonymous_author.anonymous = True
        anonymous_author.roles = [Role("Reviewer", "referee")]
        sub_article_object.contributors.append(anonymous_author)
    if sub_article_object.article_type == "author-comment":
        for author in article_object.contributors:
            if not author.roles:
                author.roles = [Role("Author", "author")]
            sub_article_object.contributors.append(author)


def build_sub_article_object(article_object, xml_root, content, index):
    # generate a DOI value and create an article object
    sub_article_object = Article(sub_article_doi(article_object.doi, index))
    # set the article id
    sub_article_object.id = sub_article_id(index)
    # set the article type
    sub_article_object.article_type = ARTICLE_TYPE_MAP.get(
        content.get("type"), content.get("type")
    )
    sub_article_contributors(article_object, sub_article_object)
    # take the article title from the XML
    article_title_tag = xml_root.find(".//front-stub/title-group/article-title")
    if article_title_tag is not None:
        # handle inline tags
        tag_text = ElementTree.tostring(article_title_tag).decode("utf8")
        # remove article-title tag
        sub_article_object.title = tag_text.replace("<article-title>", "").replace(
            "</article-title>", ""
        )
    return sub_article_object


def format_content_json(content_json, article_object):
    data = []
    # parse html to xml
    content_json = docmap_parse.transform_docmap_content(content_json)
    # reorder the articles
    content_json = reorder_content_json(content_json)
    # create an article for each
    for index, content in enumerate(content_json):

        xml_root = ElementTree.fromstring(content.get("xml"))
        sub_article_object = build_sub_article_object(
            article_object, xml_root, content, index
        )

        data.append(
            {
                "article": sub_article_object,
                "xml_root": xml_root,
            }
        )

    return data


def generate(data, root_tag="article"):
    "generate a sub-article XML tag for each article"
    root = Element(root_tag)

    for data_item in data:
        article = data_item.get("article")
        sub_article_root = data_item.get("xml_root")

        # set the article-type for each article
        sub_article_tag = sub_article(root, article.id, article.article_type)

        # front-stub parent tag
        front_stub_tag = SubElement(sub_article_tag, "front-stub")

        build.set_article_id(front_stub_tag, article)
        build.set_title_group(front_stub_tag, article)

        # add contributor tags
        if article.contributors:
            set_contrib(front_stub_tag, article)

        build.set_related_object(front_stub_tag, article)

        # set body from the sub-article XML
        body_tag = sub_article_root.find("body")
        if body_tag is not None:
            sub_article_tag.append(body_tag)

        # if editor-report, extra formatting for the abstract
        if article.article_type == "editor-report":
            assessment_terms.add_assessment_terms(sub_article_tag)

    # repair namespaces
    repair_namespaces(root)

    return root


def repair_namespaces(root):
    "repair XML namespaces by adding namespaces if missing"
    all_attributes = set()
    for tag in root.iter("*"):
        all_attributes = all_attributes.union(
            all_attributes, {attribute_name for attribute_name in tag.attrib.keys()}
        )
    prefix_attributes = {
        attrib.split(":")[0] for attrib in all_attributes if ":" in attrib
    }

    for prefix in prefix_attributes:
        if prefix in XML_NAMESPACES.keys():
            ns_attrib = "xmlns:%s" % prefix
            root.set(ns_attrib, XML_NAMESPACES.get(prefix))


def sub_article(parent, id_attribute=None, article_type=None):
    sub_article_tag = SubElement(parent, "sub-article")
    if id_attribute:
        sub_article_tag.set("id", id_attribute)
    if article_type:
        sub_article_tag.set("article-type", article_type)
    return sub_article_tag


def set_contrib(parent, article, contrib_type=None):
    contrib_group = SubElement(parent, "contrib-group")

    for contributor in article.contributors:
        contrib_tag = SubElement(contrib_group, "contrib")
        contrib_tag.set("contrib-type", contributor.contrib_type)
        build.set_contrib_name(contrib_tag, contributor)

        # set role tag
        build.set_contrib_role(contrib_tag, contrib_type, contributor)

        # set orcid tag with authenticated=true tag attribute
        build.set_contrib_orcid(contrib_tag, contributor)

        # add aff tag(s)
        for affiliation in contributor.affiliations:
            build.set_aff(
                contrib_tag,
                affiliation,
                contrib_type,
                aff_id=None,
                tail="",
                institution_wrap=True,
            )
